@@ -1,20 +1,20 @@
-id = "17484cab96a2d0eb9a30d9af5f100a63216c981e"
-date = "2014-01-27 16:12:56 +0000"
-branch = "master"
-tag = "None"
+id = "127f772a85ce01e58e3559db1487119cf4c26ec8"
+date = "2014-11-19 20:10:17 +0000"
+branch = "pylal-0.6.x"
+tag = "pylal-v0.6.0-v1"
 if tag == "None":
     tag = None
-author = "Yuanhao Zhang <yuanhao.zhang@ligo.org>"
+author = "Kipp Cannon <kipp.cannon@ligo.org>"
 builder = "Kipp Cannon <kipp.cannon@ligo.org>"
-committer = "Yuanhao Zhang <yuanhao.zhang@ligo.org>"
+committer = "Kipp Cannon <kipp.cannon@ligo.org>"
 status = "CLEAN: All modifications committed"
 version = id
-verbose_msg = """Branch: master
-Tag: None
-Id: 17484cab96a2d0eb9a30d9af5f100a63216c981e
+verbose_msg = """Branch: pylal-0.6.x
+Tag: pylal-v0.6.0-v1
+Id: 127f772a85ce01e58e3559db1487119cf4c26ec8
 
 Builder: Kipp Cannon <kipp.cannon@ligo.org>
-Build date: 2014-01-27 18:12:00 +0000
+Build date: 2014-11-19 20:18:30 +0000
 Repository status: CLEAN: All modifications committed"""
 
 import warnings
@@ -33,9 +33,9 @@ def check_match(foreign_id, onmismatch="raise"):
     """
     if onmismatch not in ("raise", "warn"):
         raise ValueError, onmismatch + " is an unrecognized value of onmismatch"
-    if foreign_id == "17484cab96a2d0eb9a30d9af5f100a63216c981e":
+    if foreign_id == "127f772a85ce01e58e3559db1487119cf4c26ec8":
         return
-    msg = "Program id (17484cab96a2d0eb9a30d9af5f100a63216c981e) does not match given id (%s)." % foreign_id
+    msg = "Program id (127f772a85ce01e58e3559db1487119cf4c26ec8) does not match given id (%s)." % foreign_id
     if onmismatch == "raise":
         raise VersionMismatchError, msg
 
