@@ -1,4 +1,4 @@
-# Copyright (C) 2006--2010  Kipp Cannon, Drew G. Keppel, Jolien Creighton
+# Copyright (C) 2006--2014  Kipp Cannon, Drew G. Keppel, Jolien Creighton
 #
 # This program is free software; you can redistribute it and/or modify it
 # under the terms of the GNU General Public License as published by the
@@ -31,26 +31,37 @@ Light Weight XML documents.
 
 
 import bisect
+try:
+	from fpconst import NaN, NegInf, PosInf
+except ImportError:
+	# fpconst is not part of the standard library and might not be
+	# available
+	NaN = float("nan")
+	NegInf = float("-inf")
+	PosInf = float("+inf")
 import itertools
 import math
 import numpy
 import random
-try:
-	from scipy.constants import c as speed_of_light
-except ImportError:
-	from lal import LAL_C_SI as speed_of_light
-	speed_of_light = float(speed_of_light)
+from scipy.constants import c as speed_of_light
 import scipy.optimize
 import sys
+import threading
+import warnings
 
 
 from glue import iterutils
 from glue import offsetvector
 from glue import segmentsUtils
-from glue.ligolw import table
+from glue.ligolw import ligolw
+from glue.ligolw import array as ligolw_array
+from glue.ligolw import param as ligolw_param
+from glue.ligolw import table as ligolw_table
 from glue.ligolw import lsctables
+from glue.text_progress_bar import ProgressBar
 from pylal import git_version
 from pylal import inject
+from pylal import rate
 
 
 __author__ = "Kipp Cannon <kipp.cannon@ligo.org>"
@@ -206,7 +217,7 @@ def make_eventlists(xmldoc, EventListType, event_table_name, process_ids = None)
 	the events, a maximum allowed time window, and the name of the
 	program that generated the events.
 	"""
-	return EventListDict(EventListType, table.get_table(xmldoc, event_table_name), process_ids = process_ids)
+	return EventListDict(EventListType, ligolw_table.get_table(xmldoc, event_table_name), process_ids = process_ids)
 
 
 #
@@ -284,8 +295,8 @@ def get_doubles(eventlists, comparefunc, instruments, thresholds, verbose = Fals
 
 	try:
 		threshold_data = thresholds[(eventlista.instrument, eventlistb.instrument)]
-	except KeyError, e:
-		raise KeyError, "no coincidence thresholds provided for instrument pair %s, %s" % e.args[0]
+	except KeyError as e:
+		raise KeyError("no coincidence thresholds provided for instrument pair %s, %s" % e.args[0])
 	light_travel_time = inject.light_travel_time(eventlista.instrument, eventlistb.instrument)
 
 	# for each event in the shortest list
@@ -604,9 +615,9 @@ class TimeSlideGraph(object):
 	def get_coincs(self, eventlists, event_comparefunc, thresholds, include_small_coincs = True, verbose = False):
 		if verbose:
 			print >>sys.stderr, "constructing coincs for target offset vectors ..."
-		for n, node in enumerate(self.head):
+		for n, node in enumerate(self.head, start = 1):
 			if verbose:
-				print >>sys.stderr, "%d/%d: %s" % (n + 1, len(self.head), str(node.offset_vector))
+				print >>sys.stderr, "%d/%d: %s" % (n, len(self.head), str(node.offset_vector))
 			if include_small_coincs:
 				# note that unused_coincs must be retrieved
 				# after the call to .get_coincs() because
@@ -739,10 +750,11 @@ class CoincSynthesizer(object):
 		features will not be available if a dictionary of counts is
 		provided).  segmentlists is a glue.segments.segmentlistdict
 		object describing the observation segments for each of the
-		instruments.  delta_t is a time window in seconds, the
-		light travel time between instrument pairs is added to this
-		internally to set the maximum allowed coincidence window
-		between a pair of instruments.
+		instruments.  A segment list must be provided for (at
+		least) each instrument in eventlists.  delta_t is a time
+		window in seconds, the light travel time between instrument
+		pairs is added to this internally to set the maximum
+		allowed coincidence window between a pair of instruments.
 
 		abundance_rel_accuracy sets the fractional error tolerated
 		in the Monte Carlo integrator used to estimate the relative
@@ -764,7 +776,7 @@ class CoincSynthesizer(object):
 		{frozenset(['V1', 'H1']): 0.0, frozenset(['V1', 'H1', 'L1']): 0.25, frozenset(['H1', 'L1']): 0.25, frozenset(['V1', 'L1']): 0.5}
 		"""
 		self.eventlists = eventlists if eventlists is not None else dict.fromkeys(segmentlists, 0) if segmentlists is not None else {}
-		self.segmentlists = segmentlists if segmentlists is not None else segments.segmentlistdict()
+		self.segmentlists = segmentlists if segmentlists is not None else segmentsUtils.segments.segmentlistdict()
 		self.delta_t = delta_t
 		# require a segment list for each list of events
 		assert set(self.eventlists) <= set(self.segmentlists)
@@ -927,9 +939,13 @@ class CoincSynthesizer(object):
 			for instruments in self.all_instrument_combos:
 		# choose the instrument whose TOA forms the "epoch" of the
 		# coinc.  to improve the convergence rate this should be
-		# the instrument with the smallest coincidence windows
+		# the instrument with the smallest Cartesian product of
+		# coincidence windows with other instruments (so that
+		# coincidence with this instrument provides the tightest
+		# prior constraint on the time differences between the
+		# other instruments).
 				key = instruments
-				anchor = min((self.tau[frozenset(ab)], ab[0]) for ab in iterutils.choices(tuple(instruments), 2))[1]
+				anchor = min(instruments, key = lambda a: sum(math.log(self.tau[frozenset((a, b))]) for b in instruments - set([a])))
 				instruments = tuple(instruments - set([anchor]))
 		# compute \mu_{1} * \mu_{2} ... \mu_{N} * 2 * \tau_{12} * 2
 		# * \tau_{13} ... 2 * \tau_{1N}.  this is the rate at which
@@ -953,6 +969,12 @@ class CoincSynthesizer(object):
 		# FIXME:  it might be practical to solve this with some
 		# sort of computational geometry library and convex hull
 		# volume calculator.
+		# FIXME:  in any case, these correction factors depend only
+		# on the coincidence windows and can be computed and saved
+		# when self.tau is updated allowing the rates, here, to be
+		# computed very quickly if only the single-instrument
+		# trigger rates have changed and not the coincidence
+		# windows.
 				if len(instruments) > 1:
 		# for each instrument 2...N, the interval within which an
 		# event is coincident with instrument 1
@@ -967,25 +989,38 @@ class CoincSynthesizer(object):
 		# also mutually coincident.  this is done by picking a
 		# vector of allowed \Delta ts and testing them against the
 		# coincidence windows.  the loop's exit criterion is
-		# arrived at as follows.  the binomial distribution's
-		# variance is d p (1 - p) where d is the number of trials
-		# and p is the probability of a successful outcome, which
-		# we replace here with p=n/d.  we quit when \sqrt{d p (1 -
-		# p)} / n <= rel accuracy.  by connecting the bailout
-		# condition to the results of the loop we bias the final
-		# answer (e.g., we get lucky and increment n on the first
-		# trial).  to minimize the effect of this we require at
-		# least 1 / rel accuracy iterations before considering the
-		# binomial criterion.  note that if the true probability is
-		# 0 or 1, so that n=0 or n=d identically then the loop will
-		# never terminate;  from the nature of the problem we know
-		# 0<p<1 so the loop will, eventually, terminate
+		# arrived at as follows.  after d trials, the number of
+		# successful outcomes is a binomially-distributed RV with
+		# variance = d p (1 - p) <= d/4 where p is the probability
+		# of a successful outcome.  we quit when the ratio of the
+		# bound on the standard deviation of the number of
+		# successful outcomes to the actual number of successful
+		# outcomes falls below rel accuracy: \sqrt{d/4} / n < rel
+		# accuracy.  note that if the true probability is 0, so
+		# that n=0 identically, then the loop will never terminate;
+		# from the nature of the problem we know 0<p<1 so the loop
+		# will, eventually, terminate.  note that if instead of
+		# using the upper bound on the variance, we replace p with
+		# (n/d) and use that estimate of the variance the loop can
+		# be shown to require many fewer iterations to meet the
+		# desired accuracy, but that choice creates a rather strong
+		# bias that, to overcome, requires some extra hacks to
+		# force the loop to run for additional iterations.  this
+		# approach is cleaner.
+					math_sqrt = math.sqrt
+					random_uniform = random.uniform
+					epsilon = self.abundance_rel_accuracy
 					n, d = 0, 0
-					while self.abundance_rel_accuracy * d < 1.0 or n < d / (1.0 + self.abundance_rel_accuracy**2 * d):
-						dt = tuple(random.uniform(*window) for window in windows)
+					while math_sqrt(d) >= epsilon * n:
+						dt = tuple(random_uniform(*window) for window in windows)
 						if all(abs(dt[i] - dt[j]) <= maxdt for i, j, maxdt in ijseq):
-							n += 1
+		# instead of adding 1 here and multiplying n by 2 in the
+		# loop exit test, we increment n by 2 and then fix it
+		# afterwards.
+							n += 2
 						d += 1
+		# fix n (see above)
+					n //= 2
 
 					rate *= float(n) / float(d)
 					if self.verbose:
@@ -1408,3 +1443,595 @@ class TOATriangulator(object):
 
 		# done
 		return n, t0 + toa, chi2 / len(self.sigmas), dt
+
+
+#
+# =============================================================================
+#
+#                     Coincidence Parameter Distributions
+#
+# =============================================================================
+#
+
+
+#
+# A look-up table used to convert instrument names to powers of 2.  Why?
+# To create a bidirectional mapping between combinations of instrument
+# names and integers so we can use a pylal.rate style binning for the
+# instrument combinations.  This has to be used because pylal.rate's native
+# Categories binning cannot be serialized to XML.
+#
+# FIXME:  allow pylal.rate's Categories binning to be serialized to XML and
+# get rid of this crap
+#
+
+
+class InstrumentCategories(dict):
+	def __init__(self):
+		# FIXME:  we decided that the coherent and null stream
+		# naming convention would look like
+		#
+		# H1H2:LSC-STRAIN_HPLUS, H1H2:LSC-STRAIN_HNULL
+		#
+		# and so on.  i.e., the +, x and null streams from a
+		# coherent network would be different channels from a
+		# single instrument whose name would be the mash-up of the
+		# names of the instruments in the network.  that is
+		# inconsisntent with the "H1H2+", "H1H2-" shown here, so
+		# this needs to be fixed but I don't know how.  maybe it'll
+		# go away before it needs to be fixed.
+		self.update(dict((instrument, 1 << n) for n, instrument in enumerate(("G1", "H1", "H2", "H1H2+", "H1H2-", "L1", "V1", "E1", "E2", "E3", "E0"))))
+
+	def max(self):
+		return sum(self.values())
+
+	def category(self, instruments):
+		return sum(self[instrument] for instrument in instruments)
+
+	def instruments(self, category):
+		return set(instrument for instrument, factor in self.items() if category & factor)
+
+
+#
+# A class for measuring parameter distributions
+#
+
+
+class CoincParamsDistributions(object):
+	"""
+	A class for histograming the parameters of coincidences (or of
+	single events).  It is assumed there is a fixed, pre-determined,
+	set of parameters that one wishes to histogram, and that each
+	parameter has a name.  To use this, it must be sub-classed and the
+	derived class must provide dictionaries of binnings and functions
+	for performing the kernel density estimation transform to obtain
+	PDFs from histograms of counts.  The binnings is a dictionary
+	mapping parameter names to rate.NDBins instances describing the
+	binning to be used for each paramter.  The pdf_from_rates_func
+	dictionary maps parameter names to functions to smooth and
+	normalize bin count data into PDFs.  As a special case, a default
+	function is provided and will be used for any parameters whose
+	names do not appear in the pdf_from_rates_func dictionary.  The
+	default function looks for a smoothing filter in the filters
+	dictionary, applies it if found, then invokes the .to_pdf() method
+	of the binned array object.  Subclasses must also provide a
+	.coinc_params() static method that will transform a list of
+	single-instrument events into a dictionary mapping paramter name to
+	parameter value.
+
+	This class maintains three sets of histograms, one set for noise
+	(or "background") events, one set for signal (or "injection")
+	events and one set for observed (or "zero lag") events.  The bin
+	counts are floating point values (not integers).
+	"""
+	#
+	# sub-classes may override the following
+	#
+
+	ligo_lw_name_suffix = u"pylal_snglcoinc_coincparamsdistributions"
+
+	#
+	# Default content handler for loading CoincParamsDistributions
+	# objects from XML documents
+	#
+
+	class LIGOLWContentHandler(ligolw.LIGOLWContentHandler):
+		pass
+	ligolw_array.use_in(LIGOLWContentHandler)
+	lsctables.use_in(LIGOLWContentHandler)
+	ligolw_param.use_in(LIGOLWContentHandler)
+
+	#
+	# sub-classes must override the following
+	#
+
+	binnings = {}
+
+	pdf_from_rates_func = {}
+	filters = {}
+
+	@staticmethod
+	def coinc_params(*args, **kwargs):
+		"""
+		Given a sequence of single-instrument events (rows from an
+		event table) that form a coincidence, compute and return a
+		dictionary mapping parameter name to parameter values,
+		suitable for being passed to one of the .add_*() methods.
+		This function may return None.
+		"""
+		raise NotImplementedError("subclass must implement .coinc_params() method")
+
+	#
+	# begin implementation
+	#
+
+	def __init__(self, process_id = None):
+		if not self.binnings:
+			raise NotImplementedError("subclass must provide dictionary of binnings")
+		self.zero_lag_rates = dict((param, rate.BinnedArray(binning)) for param, binning in self.binnings.items())
+		self.background_rates = dict((param, rate.BinnedArray(binning)) for param, binning in self.binnings.items())
+		self.injection_rates = dict((param, rate.BinnedArray(binning)) for param, binning in self.binnings.items())
+		self.zero_lag_pdf = {}
+		self.background_pdf = {}
+		self.injection_pdf = {}
+		self.zero_lag_lnpdf_interp = {}
+		self.background_lnpdf_interp = {}
+		self.injection_lnpdf_interp = {}
+		self.process_id = process_id
+
+	def _rebuild_interpolators(self):
+		"""
+		Initialize the interp dictionaries from the discretely
+		sampled PDF data.  For internal use only.
+		"""
+		self.zero_lag_lnpdf_interp.clear()
+		self.background_lnpdf_interp.clear()
+		self.injection_lnpdf_interp.clear()
+		def mkinterp(binnedarray):
+			assert not (binnedarray.array < 0.).any()
+			binnedarray = binnedarray.copy()
+			with numpy.errstate(divide = "ignore"):
+				binnedarray.array = numpy.log(binnedarray.array)
+			return rate.InterpBinnedArray(binnedarray, fill_value = NegInf)
+		for key, binnedarray in self.zero_lag_pdf.items():
+			self.zero_lag_lnpdf_interp[key] = mkinterp(binnedarray)
+		for key, binnedarray in self.background_pdf.items():
+			self.background_lnpdf_interp[key] = mkinterp(binnedarray)
+		for key, binnedarray in self.injection_pdf.items():
+			self.injection_lnpdf_interp[key] = mkinterp(binnedarray)
+
+	@staticmethod
+	def addbinnedarrays(rate_target_dict, rate_source_dict, pdf_target_dict, pdf_source_dict):
+		"""
+		For internal use.
+		"""
+		weight_target = {}
+		weight_source = {}
+		for name, binnedarray in rate_source_dict.items():
+			if name in rate_target_dict:
+				weight_target[name] = rate_target_dict[name].array.sum()
+				weight_source[name] = rate_source_dict[name].array.sum()
+				rate_target_dict[name] += binnedarray
+			else:
+				rate_target_dict[name] = binnedarray.copy()
+		for name, binnedarray in pdf_source_dict.items():
+			if name in pdf_target_dict:
+				binnedarray = binnedarray.copy()
+				binnedarray.array *= weight_source[name]
+				pdf_target_dict[name].array *= weight_target[name]
+				pdf_target_dict[name] += binnedarray
+				pdf_target_dict[name].array /= weight_source[name] + weight_target[name]
+			else:
+				pdf_target_dict[name] = binnedarray.copy()
+
+	def __iadd__(self, other):
+		if type(other) != type(self):
+			raise TypeError(other)
+
+		self.addbinnedarrays(self.zero_lag_rates, other.zero_lag_rates, self.zero_lag_pdf, other.zero_lag_pdf)
+		self.addbinnedarrays(self.background_rates, other.background_rates, self.background_pdf, other.background_pdf)
+		self.addbinnedarrays(self.injection_rates, other.injection_rates, self.injection_pdf, other.injection_pdf)
+
+		#
+		# rebuild interpolators
+		#
+
+		self._rebuild_interpolators()
+
+		#
+		# done
+		#
+
+		return self
+
+	def copy(self):
+		new = type(self)(process_id = self.process_id)
+		new += self
+		return new
+
+	def add_zero_lag(self, param_dict, weight = 1.0):
+		"""
+		Increment a bin in one or more of the observed data (or
+		"zero lag") histograms by weight (default 1).  The names of
+		the histograms to increment, and the parameters identifying
+		the bin in each histogram, are given by the param_dict
+		dictionary.
+		"""
+		for param, value in param_dict.items():
+			try:
+				self.zero_lag_rates[param][value] += weight
+			except IndexError:
+				# param value out of range
+				pass
+
+	def add_background(self, param_dict, weight = 1.0):
+		"""
+		Increment a bin in one or more of the noise (or
+		"background") histograms by weight (default 1).  The names
+		of the histograms to increment, and the parameters
+		identifying the bin in each histogram, are given by the
+		param_dict dictionary.
+		"""
+		for param, value in param_dict.items():
+			try:
+				self.background_rates[param][value] += weight
+			except IndexError:
+				# param value out of range
+				pass
+
+	def add_injection(self, param_dict, weight = 1.0):
+		"""
+		Increment a bin in one or more of the signal (or
+		"injection") histograms by weight (default 1).  The names
+		of the histograms to increment, and the parameters
+		identifying the bin in each histogram, are given by the
+		param_dict dictionary.
+		"""
+		for param, value in param_dict.items():
+			try:
+				self.injection_rates[param][value] += weight
+			except IndexError:
+				# param value out of range
+				pass
+
+	def default_pdf_from_rates(self, key, pdf_dict):
+		"""
+		For internal use by the CoincParamsDistributions class.
+		"""
+		binnedarray = pdf_dict[key]
+		if key in self.filters:
+			rate.filter_array(binnedarray.array, self.filters[key])
+		binnedarray.to_pdf()
+
+	def finish(self, verbose = False):
+		"""
+		Populate the discrete PDF dictionaries from the contents of
+		the rates dictionaries, and then the PDF interpolator
+		dictionaries from the discrete PDFs.  The raw bin counts
+		from the rates dictionaries are copied verbatim, smoothed
+		using the dictionary of filters carried by this class
+		instance, and converted to normalized PDFs using the bin
+		volumes.  Finally the dictionary of PDF interpolators is
+		populated from the discretely sampled PDF data.
+		"""
+		#
+		# convert raw bin counts into normalized PDFs
+		#
+
+		self.zero_lag_pdf.clear()
+		self.background_pdf.clear()
+		self.injection_pdf.clear()
+		progressbar = ProgressBar(text = "Computing Parameter PDFs", max = len(self.zero_lag_rates) + len(self.background_rates) + len(self.injection_rates)) if verbose else None
+		for key, (msg, rates_dict, pdf_dict) in itertools.chain(
+				zip(self.zero_lag_rates, itertools.repeat(("zero lag", self.zero_lag_rates, self.zero_lag_pdf))),
+				zip(self.background_rates, itertools.repeat(("background", self.background_rates, self.background_pdf))),
+				zip(self.injection_rates, itertools.repeat(("injections", self.injection_rates, self.injection_pdf)))
+		):
+			assert numpy.isfinite(rates_dict[key].array).all() and (rates_dict[key].array >= 0).all(), "%s %s counts are not valid" % (key, msg)
+			pdf_dict[key] = rates_dict[key].copy()
+			try:
+				pdf_from_rates_func = self.pdf_from_rates_func[key]
+			except KeyError:
+				pdf_from_rates_func = self.default_pdf_from_rates
+			if pdf_from_rates_func is not None:
+				pdf_from_rates_func(key, pdf_dict)
+			if progressbar is not None:
+				progressbar.increment()
+
+		#
+		# rebuild interpolators
+		#
+
+		self._rebuild_interpolators()
+
+	def lnP_noise(self, params):
+		"""
+		From a parameter value dictionary as returned by
+		self.coinc_params(), compute and return the natural
+		logarithm of the noise probability density at that point in
+		parameter space.
+
+		The .finish() method must have been invoked before this
+		method does meaningful things.  No attempt is made to
+		ensure the .finish() method has been invoked nor, if it has
+		been invoked, that no manipulations have occured that might
+		require it to be re-invoked (e.g., the contents of the
+		parameter distributions have been modified and require
+		re-normalization).
+
+		This default implementation assumes the individual PDFs
+		containined in the noise dictionary are for
+		statistically-independent random variables, and computes
+		and returns the logarithm of their product.  Sub-classes
+		that require more sophisticated calculations can override
+		this method.
+		"""
+		__getitem__ = self.background_lnpdf_interp.__getitem__
+		return sum(__getitem__(name)(*value) for name, value in params.items())
+
+	def lnP_signal(self, params):
+		"""
+		From a parameter value dictionary as returned by
+		self.coinc_params(), compute and return the natural
+		logarithm of the signal probability density at that point
+		in parameter space.
+
+		The .finish() method must have been invoked before this
+		method does meaningful things.  No attempt is made to
+		ensure the .finish() method has been invoked nor, if it has
+		been invoked, that no manipulations have occured that might
+		require it to be re-invoked (e.g., the contents of the
+		parameter distributions have been modified and require
+		re-normalization).
+
+		This default implementation assumes the individual PDFs
+		containined in the signal dictionary are for
+		statistically-independent random variables, and computes
+		and returns the logarithm of their product.  Sub-classes
+		that require more sophisticated calculations can override
+		this method.
+		"""
+		__getitem__ = self.injection_lnpdf_interp.__getitem__
+		return sum(__getitem__(name)(*value) for name, value in params.items())
+
+	def get_xml_root(self, xml, name):
+		"""
+		Sub-classes can use this in their overrides of the
+		.from_xml() method to find the root element of the XML
+		serialization.
+		"""
+		name = u"%s:%s" % (name, self.ligo_lw_name_suffix)
+		xml = [elem for elem in xml.getElementsByTagName(ligolw.LIGO_LW.tagName) if elem.hasAttribute(u"Name") and elem.Name == name]
+		if len(xml) != 1:
+			raise ValueError("XML tree must contain exactly one %s element named %s" % (ligolw.LIGO_LW.tagName, name))
+		return xml[0]
+
+	@classmethod
+	def from_xml(cls, xml, name):
+		"""
+		In the XML document tree rooted at xml, search for the
+		serialized CoincParamsDistributions object named name, and
+		deserialize it.  The return value is a two-element tuple.
+		The first element is the deserialized
+		CoincParamsDistributions object, the second is the process
+		ID recorded when it was written to XML.
+		"""
+		# create an instance
+		self = cls()
+
+		# find the root element of the XML serialization
+		xml = self.get_xml_root(xml, name)
+
+		# retrieve the process ID
+		self.process_id = ligolw_param.get_pyvalue(xml, u"process_id")
+
+		# reconstruct the BinnedArray objects
+		def reconstruct(xml, prefix, target_dict):
+			for name in [elem.Name.split(u":")[1] for elem in xml.childNodes if elem.Name.startswith(u"%s:" % prefix)]:
+				target_dict[str(name)] = rate.BinnedArray.from_xml(xml, u"%s:%s" % (prefix, name))
+		reconstruct(xml, u"zero_lag", self.zero_lag_rates)
+		reconstruct(xml, u"zero_lag_pdf", self.zero_lag_pdf)
+		reconstruct(xml, u"background", self.background_rates)
+		reconstruct(xml, u"background_pdf", self.background_pdf)
+		reconstruct(xml, u"injection", self.injection_rates)
+		reconstruct(xml, u"injection_pdf", self.injection_pdf)
+
+		#
+		# rebuild interpolators
+		#
+
+		self._rebuild_interpolators()
+
+		#
+		# done
+		#
+
+		return self
+
+	def to_xml(self, name):
+		"""
+		Serialize this CoincParamsDistributions object to an XML
+		fragment and return the root element of the resulting XML
+		tree.  The .process_id attribute of process will be
+		recorded in the serialized XML, and the object will be
+		given the name name.
+		"""
+		xml = ligolw.LIGO_LW({u"Name": u"%s:%s" % (name, self.ligo_lw_name_suffix)})
+		xml.appendChild(ligolw_param.new_param(u"process_id", u"ilwd:char", self.process_id))
+		def store(xml, prefix, source_dict):
+			for name, binnedarray in sorted(source_dict.items()):
+				xml.appendChild(binnedarray.to_xml(u"%s:%s" % (prefix, name)))
+		store(xml, u"zero_lag", self.zero_lag_rates)
+		store(xml, u"zero_lag_pdf", self.zero_lag_pdf)
+		store(xml, u"background", self.background_rates)
+		store(xml, u"background_pdf", self.background_pdf)
+		store(xml, u"injection", self.injection_rates)
+		store(xml, u"injection_pdf", self.injection_pdf)
+
+		return xml
+
+	@classmethod
+	def from_filenames(cls, filenames, name, verbose = False, contenthandler = None):
+		"""
+		Convenience function to deserialize
+		CoincParamsDistributions objects from a collection of XML
+		files and return their sum.  The return value is a
+		two-element tuple.  The first element is the deserialized
+		and summed CoincParamsDistributions object, the second is a
+		segmentlistdict indicating the interval of time spanned by
+		the out segments in the search_summary rows matching the
+		process IDs that were attached to the
+		CoincParamsDistributions objects in the XML.
+		"""
+		if contenthandler is None:
+			contenthandler = cls.LIGOLWContentHandler
+		self = None
+		for n, filename in enumerate(filenames):
+			if verbose:
+				print >>sys.stderr, "%d/%d:" % (n + 1, len(filenames)),
+			xmldoc = utils.load_filename(filename, verbose = verbose, contenthandler = contenthandler)
+			if self is None:
+				self = cls.from_xml(xmldoc, name)
+				seglists = lsctables.SearchSummaryTable.get_table(xmldoc).get_out_segmentlistdict(set([self.process_id])).coalesce()
+			else:
+				other = cls.from_xml(xmldoc, name)
+				self += other
+				seglists |= lsctables.SearchSummaryTable.get_table(xmldoc).get_out_segmentlistdict(set([other.process_id])).coalesce()
+				del other
+			xmldoc.unlink()
+		return self, seglists
+
+
+#
+# Likelihood Ratio
+#
+
+
+# starting from Bayes' theorem:
+#
+# P(coinc is a g.w. | its parameters)
+#     P(those parameters | a coinc known to be a g.w.) * P(coinc is g.w.)
+#   = -------------------------------------------------------------------
+#                                P(parameters)
+#
+#     P(those parameters | a coinc known to be a g.w.) * P(coinc is g.w.)
+#   = -------------------------------------------------------------------
+#     P(noise params) * P(coinc is not g.w.) + P(inj params) * P(coinc is g.w.)
+#
+#                       P(inj params) * P(coinc is g.w.)
+#   = -------------------------------------------------------------------
+#     P(noise params) * [1 - P(coinc is g.w.)] + P(inj params) * P(coinc is g.w.)
+#
+#                        P(inj params) * P(coinc is g.w.)
+#   = ----------------------------------------------------------------------
+#     P(noise params) + [P(inj params) - P(noise params)] * P(coinc is g.w.)
+#
+# this last form above is used below to compute the LHS
+#
+#          [P(inj params) / P(noise params)] * P(coinc is g.w.)
+#   = --------------------------------------------------------------
+#     1 + [[P(inj params) / P(noise params)] - 1] * P(coinc is g.w.)
+#
+#          Lambda * P(coinc is g.w.)                       P(inj params)
+#   = -----------------------------------  where Lambda = ---------------
+#     1 + (Lambda - 1) * P(coinc is g.w.)                 P(noise params)
+#
+# Differentiating w.r.t. Lambda shows the derivative is always positive, so
+# thresholding on Lambda is equivalent to thresholding on P(coinc is a g.w.
+# | its parameters).  The limits:  Lambda=0 --> P(coinc is a g.w. | its
+# parameters)=0, Lambda=+inf --> P(coinc is a g.w. | its parameters)=1.  We
+# interpret Lambda=0/0 to mean P(coinc is a g.w. | its parameters)=0 since
+# although it can't be noise it's definitely not a g.w..  We do not protect
+# against NaNs in the Lambda = +inf/+inf case.
+
+
+class LnLikelihoodRatio(object):
+	"""
+	Class for computing signal hypothesis / noise hypothesis likelihood
+	ratios from the measurements in a
+	snglcoinc.CoincParamsDistributions instance.
+	"""
+	def __init__(self, coinc_param_distributions):
+		self.lnP_noise = coinc_param_distributions.lnP_noise
+		self.lnP_signal = coinc_param_distributions.lnP_signal
+
+	def __call__(self, *args, **kwargs):
+		"""
+		Return the natural logarithm of the likelihood ratio for
+		the hypothesis that the list of events are the result of a
+		gravitational wave.  The likelihood ratio is the ratio
+		P(inj params) / P(noise params).  The probability that the
+		events are the result of a gravitiational wave is a
+		monotonically increasing function of the likelihood ratio,
+		so ranking events from "most like a gravitational wave" to
+		"least like a gravitational wave" can be performed by
+		calculating the (logarithm of the) likelihood ratios, which
+		has the advantage of not requiring a prior probability to
+		be provided (knowing how many gravitational waves you've
+		actually detected).
+
+		The arguments are passed verbatim to the .lnP_noise and
+		.lnP_signal() methods of the
+		snglcoinc.CoincParamsDistributions instance with which this
+		object is associated.
+		"""
+		lnP_noise = self.lnP_noise(*args, **kwargs)
+		lnP_signal = self.lnP_signal(*args, **kwargs)
+		if math.isinf(lnP_noise) and math.isinf(lnP_signal):
+			# need to handle a special case
+			if lnP_noise < 0. and lnP_signal < 0.:
+				# both probabilities are 0.  "correct"
+				# answer is -inf, because if a candidate is
+				# in a region of parameter space where the
+				# probability of a signal occuring is 0
+				# then there is no way it is a signal.
+				# there is also, aparently, no way it's a
+				# noise event, which is puzzling, but
+				# that's irrelevant because we are supposed
+				# to be computing something that is a
+				# monotonically increasing function of the
+				# probability that a candidate is a signal,
+				# which is 0 in this part of the parameter
+				# space.
+				return NegInf
+			# all remaining cases are handled correctly by the
+			# expression that follows, but one still deserves a
+			# warning
+			if lnP_noise > 0. and lnP_signal > 0.:
+				# both probabilities are +inf.  no correct
+				# answer.
+				warnings.warn("inf/inf encountered")
+		return  lnP_signal - lnP_noise
+
+	def samples(self, random_params_seq, **kwargs):
+		"""
+		Generator that yields an unending sequence of 3-element
+		tuples.  Each tuple's elements are a value of the natural
+		logarithm of the likelihood rato, the natural logarithm of
+		the probability density of that likelihood ratio in the
+		signal population, the natural logarithm of the probability
+		density of that likelihood ratio in the noise population.
+
+		random_params_seq should be a sequence (or generator) that
+		yielding 2-element tuples whose first element is a choice
+		of parameter values and whose second element is the natural
+		logarithm of the probability density from which the
+		parameters have been drawn evaluated at the parameters.
+
+		The parameter values yielded by the random_params_seq are
+		passed as the first argument, verbatim, to the .lnP_noise()
+		an .lnP_signal() methods of the CoincParamsDistributions
+		object with which this object is associated, followed by
+		any (optional) key-word arguments.
+		"""
+		lnP_noise_func = self.lnP_noise
+		lnP_signal_func = self.lnP_signal
+		isinf = math.isinf
+		for params, lnP_params in random_params_seq:
+			lnP_noise = lnP_noise_func(params, **kwargs)
+			lnP_signal = lnP_signal_func(params, **kwargs)
+			# see above for description of special cases
+			if isinf(lnP_noise) and isinf(lnP_signal) and lnP_noise < 0. and lnP_signal < 0.:
+				yield NegInf, lnP_signal - lnP_params, lnP_noise - lnP_params
+			else:
+				yield lnP_signal - lnP_noise, lnP_signal - lnP_params, lnP_noise - lnP_params
