@@ -34,6 +34,7 @@ from glue import segmentsUtils
 from glue.ligolw import lsctables
 from glue.ligolw import utils
 from glue.ligolw.utils import process as ligolw_process
+from glue.offsetvector import offsetvector
 from pylal import ligolw_burca_tailor
 from pylal import git_version
 from pylal import inject
@@ -91,123 +92,21 @@ def triangulators(timing_uncertainties):
 	return triangulators
 
 
-#
-# A look-up table used to convert instrument names to powers of 2.  Why?
-# To create a bidirectional mapping between combinations of instrument
-# names and integers so we can use a pylal.rate style binning for the
-# instrument combinations
-#
-
-
-instrument_to_factor = dict((instrument, int(2**n)) for n, instrument in enumerate(("G1", "H1", "H2", "H+", "H-", "L1", "V1")))
-
-
-def instruments_to_category(instruments):
-	return sum(instrument_to_factor[instrument] for instrument in instruments)
-
-
-def category_to_instruments(category):
-	return set(instrument for instrument, factor in instrument_to_factor.items() if category & factor)
-
-
-#
-# Coinc params function
-#
-
-
-def coinc_params_func(events, offsetvector, triangulators):
-	#
-	# check for coincs that have been vetoed entirely
-	#
-
-	if len(events) < 2:
-		return None
-
-	#
-	# Initialize the parameter dictionary, sort the events by
-	# instrument name (the multi-instrument parameters are defined for
-	# the instruments in this order and the triangulators are
-	# constructed this way too), and retrieve the sorted instrument
-	# names
-	#
-
-	params = {}
-	events = tuple(sorted(events, key = lambda event: event.ifo))
-	instruments = tuple(event.ifo for event in events)
-
-	#
-	# zero-instrument parameters
-	#
-
-	ignored, ignored, ignored, rss_timing_residual = triangulators[instruments](tuple(event.get_peak() + offsetvector[event.ifo] for event in events))
-	# FIXME:  rss_timing_residual is forced to 0 to disable this
-	# feature.  all the code to compute it properly is still here and
-	# given suitable initializations, the distribution data is still
-	# two-dimensional and has a suitable filter applied to it, but all
-	# events are forced into the RSS_{\Delta t} = 0 bin, in effect
-	# removing that dimension from the data.  We can look at this again
-	# sometime in the future if we're curious why it didn't help.  Just
-	# delete the next line and you're back in business.
-	rss_timing_residual = 0.0
-	params["instrumentgroup,rss_timing_residual"] = (instruments_to_category(instruments), rss_timing_residual)
-
-	#
-	# one-instrument parameters
-	#
-
-	for event in events:
-		prefix = "%s_" % event.ifo
-
-		params["%ssnr2_chi2" % prefix] = (event.snr**2.0, event.chisq / event.chisq_dof)
-
-	#
-	# two-instrument parameters.  note that events are sorted by
-	# instrument
-	#
-
-	for event1, event2 in iterutils.choices(events, 2):
-		assert event1.ifo != event2.ifo
-
-		prefix = "%s_%s_" % (event1.ifo, event2.ifo)
-
-		dt = float((event1.get_peak() + offsetvector[event1.ifo]) - (event2.get_peak() + offsetvector[event2.ifo]))
-		params["%sdt" % prefix] = (dt,)
-
-		dA = math.log10(abs(event1.amplitude / event2.amplitude))
-		params["%sdA" % prefix] = (dA,)
-
-		# f_cut = central_freq + bandwidth/2
-		f_cut1 = event1.central_freq + event1.bandwidth / 2
-		f_cut2 = event2.central_freq + event2.bandwidth / 2
-		df = float((math.log10(f_cut1) - math.log10(f_cut2)) / (math.log10(f_cut1) + math.log10(f_cut2)))
-		params["%sdf" % prefix] = (df,)
-
-	#
-	# done
-	#
-
-	return params
-
-
 #
 # Parameter distributions
 #
 
 
-class StringCoincParamsDistributions(ligolw_burca_tailor.BurcaCoincParamsDistributions):
-	ligo_lw_name_suffix = u"pylal_ligolw_burca_tailor_coincparamsdistributions"
-
-
 def dt_binning(instrument1, instrument2):
 	dt = 0.005 + inject.light_travel_time(instrument1, instrument2)	# seconds
 	return rate.NDBins((rate.ATanBins(-dt, +dt, 801),))
 
 
-class DistributionsStats(object):
-	"""
-	A class used to populate a StringCoincParamsDistributions instance with
-	the data from the outputs of ligolw_burca and ligolw_binjfind.
-	"""
+class StringCoincParamsDistributions(snglcoinc.CoincParamsDistributions):
+	# FIXME:  switch to new default when possible
+	ligo_lw_name_suffix = u"pylal_ligolw_burca_tailor_coincparamsdistributions"
+
+	instrument_categories = snglcoinc.InstrumentCategories()
 
 	binnings = {
 		"H1_snr2_chi2": rate.NDBins((rate.ATanLogarithmicBins(10, 1e7, 801), rate.ATanLogarithmicBins(.1, 1e4, 801))),
@@ -237,7 +136,7 @@ class DistributionsStats(object):
 		# want a binning that's linear at the origin so instead of
 		# inventing a new one we just use atan bins that are
 		# symmetric about 0
-		"instrumentgroup,rss_timing_residual": rate.NDBins((rate.LinearBins(0.5, sum(instrument_to_factor.values()) + 0.5, sum(instrument_to_factor.values())), rate.ATanBins(-0.02, +0.02, 1001)))
+		"instrumentgroup,rss_timing_residual": rate.NDBins((rate.LinearBins(0.5, instrument_categories.max() + 0.5, instrument_categories.max()), rate.ATanBins(-0.02, +0.02, 1001)))
 	}
 
 	filters = {
@@ -268,19 +167,81 @@ class DistributionsStats(object):
 		"instrumentgroup,rss_timing_residual": rate.gaussian_window(1e-100, 11, sigma = 20)
 	}
 
-	def __init__(self):
-		self.distributions = StringCoincParamsDistributions(**self.binnings)
+	@staticmethod
+	def coinc_params(events, offsetvector, triangulators):
+		#
+		# check for coincs that have been vetoed entirely
+		#
 
-	def add_noninjections(self, param_func, database, param_func_args = ()):
-		# iterate over burst<-->burst coincs
-		for is_background, events, offsetvector in ligolw_burca_tailor.get_noninjections(database):
-			events = [event for event in events if event.ifo not in database.vetoseglists or event.get_peak() not in database.vetoseglists[event.ifo]]
-			if is_background:
-				self.distributions.add_background(param_func(events, offsetvector, *param_func_args))
-			else:
-				self.distributions.add_zero_lag(param_func(events, offsetvector, *param_func_args))
+		if len(events) < 2:
+			return None
 
-	def add_slidelessbackground(self, param_func, database, experiments, param_func_args = ()):
+		#
+		# Initialize the parameter dictionary, sort the events by
+		# instrument name (the multi-instrument parameters are defined for
+		# the instruments in this order and the triangulators are
+		# constructed this way too), and retrieve the sorted instrument
+		# names
+		#
+
+		params = {}
+		events = tuple(sorted(events, key = lambda event: event.ifo))
+		instruments = tuple(event.ifo for event in events)
+
+		#
+		# zero-instrument parameters
+		#
+
+		ignored, ignored, ignored, rss_timing_residual = triangulators[instruments](tuple(event.get_peak() + offsetvector[event.ifo] for event in events))
+		# FIXME:  rss_timing_residual is forced to 0 to disable this
+		# feature.  all the code to compute it properly is still here and
+		# given suitable initializations, the distribution data is still
+		# two-dimensional and has a suitable filter applied to it, but all
+		# events are forced into the RSS_{\Delta t} = 0 bin, in effect
+		# removing that dimension from the data.  We can look at this again
+		# sometime in the future if we're curious why it didn't help.  Just
+		# delete the next line and you're back in business.
+		rss_timing_residual = 0.0
+		params["instrumentgroup,rss_timing_residual"] = (StringCoincParamsDistributions.instrument_categories.category(instruments), rss_timing_residual)
+
+		#
+		# one-instrument parameters
+		#
+
+		for event in events:
+			prefix = "%s_" % event.ifo
+
+			params["%ssnr2_chi2" % prefix] = (event.snr**2.0, event.chisq / event.chisq_dof)
+
+		#
+		# two-instrument parameters.  note that events are sorted by
+		# instrument
+		#
+
+		for event1, event2 in iterutils.choices(events, 2):
+			assert event1.ifo != event2.ifo
+
+			prefix = "%s_%s_" % (event1.ifo, event2.ifo)
+
+			dt = float((event1.get_peak() + offsetvector[event1.ifo]) - (event2.get_peak() + offsetvector[event2.ifo]))
+			params["%sdt" % prefix] = (dt,)
+
+			dA = math.log10(abs(event1.amplitude / event2.amplitude))
+			params["%sdA" % prefix] = (dA,)
+
+			# f_cut = central_freq + bandwidth/2
+			f_cut1 = event1.central_freq + event1.bandwidth / 2
+			f_cut2 = event2.central_freq + event2.bandwidth / 2
+			df = float((math.log10(f_cut1) - math.log10(f_cut2)) / (math.log10(f_cut1) + math.log10(f_cut2)))
+			params["%sdf" % prefix] = (df,)
+
+		#
+		# done
+		#
+
+		return params
+
+	def add_slidelessbackground(self, database, experiments, param_func_args = ()):
 		# FIXME:  this needs to be taught how to not slide H1 and
 		# H2 with respect to each other
 
@@ -303,7 +264,7 @@ class DistributionsStats(object):
 		# parse the --thresholds H1,L1=... command-line options from burca
 		delta_t = [float(threshold.split("=")[-1]) for threshold in ligolw_process.get_process_params(database.xmldoc, "ligolw_burca", "--thresholds")]
 		if not all(delta_t[0] == threshold for threshold in delta_t[1:]):
-			raise ValueError, "\Delta t is not unique in ligolw_burca arguments"
+			raise ValueError("\Delta t is not unique in ligolw_burca arguments")
 		delta_t = delta_t.pop()
 
 		# construct the coinc generator.  note that H1+H2-only
@@ -329,7 +290,7 @@ class DistributionsStats(object):
 		n_coincs, = scipy.stats.poisson.rvs(float(abs(segmentsUtils.vote(seglists.values(), 2))) * sum(coinc_generator.rates.values()) * experiments)
 
 		# generate synthetic background coincs
-		zero_lag_offset_vector = dict((instrument, 0.0) for instrument in seglists)
+		zero_lag_offset_vector = offsetvector((instrument, 0.0) for instrument in seglists)
 		for n, events in enumerate(coinc_generator.coincs(lsctables.SnglBurst.get_peak)):
 			# n = 1 on 2nd iteration, so placing this condition
 			# where it is in the loop causes the correct number
@@ -341,22 +302,12 @@ class DistributionsStats(object):
 			for event in events:
 				event.set_peak(toas[event.ifo])
 			# compute coincidence parameters
-			self.distributions.add_background(param_func(events, zero_lag_offset_vector, *param_func_args))
+			self.add_background(self.coinc_params(events, zero_lag_offset_vector, *param_func_args))
 
 		# restore original peak times
 		for event, peak_time in orig_peak_times.iteritems():
 			event.set_peak(peak_time)
 
-	def add_injections(self, param_func, database, weight_func = lambda sim: 1.0, param_func_args = ()):
-		# iterate over burst<-->burst coincs matching injections
-		# "exactly"
-		for sim, events, offsetvector in ligolw_burca_tailor.get_injections(database):
-			events = [event for event in events if event.ifo not in database.vetoseglists or event.get_peak() not in database.vetoseglists[event.ifo]]
-			self.distributions.add_injection(param_func(events, offsetvector, *param_func_args), weight = weight_func(sim))
-
-	def finish(self):
-		self.distributions.finish(filters = self.filters)
-
 
 #
 # I/O
@@ -364,7 +315,7 @@ class DistributionsStats(object):
 
 
 def load_likelihood_data(filenames, verbose = False):
-	return ligolw_burca_tailor.load_likelihood_data(filenames, StringCoincParamsDistributions, name = u"string_cusp_likelihood", verbose = verbose)
+	return StringCoincParamsDistributions.from_filenames(filenames, name = u"string_cusp_likelihood", verbose = verbose)
 
 
 def write_likelihood_data(filename, coincparamsdistributions, seglists, verbose = False):
